
const defaults = {
  default: {
    order: ""
  },
  selection: {
    width: 48,
    minWidth: 48,
    realWidth: 48,
    order: "",
    className: ""
  },
  expand: {
    width: 48,
    minWidth: 48,
    realWidth: 48,
    order: ""
  },
  index: {
    width: 48,
    minWidth: 48,
    realWidth: 48,
    order: ""
  },
  operation: {
    order: ""
  }
};

const objectAssign = function(target) {
  for (let i = 1, j = arguments.length; i < j; i++) {
    const source = arguments[i] || {};
    for (const prop in source) {
      if (source.hasOwnProperty(prop)) {
        const value = source[prop];
        if (value !== undefined) {
          target[prop] = value;
        }
      }
    }
  }
  return target;
};

//获取所有的列
const columns = function (slots) {
  const columns = [];
  const columnComponents = slots.filter((column) => column.componentOptions).map((column) => {
    const newColumn = {
      renderCell: column.data.scopedSlots || null,
      type: "default"
    };
    Object.assign(newColumn, column.componentOptions.propsData);
    objectAssign(newColumn, defaults[newColumn.type || "default"]);

    columns.push(newColumn);
    return column;
  });
  return columns;
};

const getDefaultOptions = (column, params = {}) => {
  // const type = column.type;

  const width = parseWidth(column.width);
  const minWidth = parseMinWidth(column.minWidth);
  const options = {
    id: column.columnId,
    columnKey: column.columnKey,
    label: column.label,
    className: column.className,
    labelClassName: column.labelClassName,
    prop: column.prop,
    property: column.prop || column.property,
    propType: column.propType,
    options: column.options,
    props: column.props,
    filterable: column.filterable,
    onlyShow: column.onlyShow,
    clearable: column.clearable,
    maxlength: column.maxlength,
    minlength: column.minlength,
    max: column.max,
    min: column.min,
    placeholder: column.placeholder,
    remote: column.remote,
    remoteMethod: column.remoteMethod,
    format: column.format,
    valueFormat: column.valueFormat,
    filterOn: column.filterOn,
    hidden: column.hidden,
    disabled: column.disabled,
    editable: column.editable,
    rightAppearance: column.rightAppearance,
    if: column.if,
    selectionVisible: column.selectionVisible,

    width: width,
    minWidth: minWidth,
    maxWidth: column.maxWidth,

    renderCell: null,
    renderHeader: column.renderHeader,
    sortable: column.sortable,
    sortMethod: column.sortMethod,
    sortBy: column.sortBy,
    resizable: column.resizable,
    context: column.context,
    align: column.align,
    headerAlign: column.headerAlign,
    showTooltipWhenOverflow: column.showTooltipWhenOverflow,
    showOverflowTooltip: column.showOverflowTooltip,
    fixed: column.fixed,
    formatter: column.formatter,
    selectable: column.selectable,
    reserveSelection: column.reserveSelection,
    filterMethod: column.filterMethod,
    filteredValue: column.filteredValue,
    filters: column.filters,
    filterPlacement: column.filterPlacement,
    filterMultiple: column.filterMultiple,
    index: column.index,
    sortOrders: column.sortOrders
  };
  Object.assign(options, params);
  return options;
};

//获取当前列信息
const getDefaultColumn = function(type, options) {
  const column = {};
  objectAssign(column, defaults[type || "default"]);
  for (const name in options) {
    if (options.hasOwnProperty(name)) {
      const value = options[name];
      if (typeof value !== "undefined") {
        column[name] = value;
      }
    }
  }
  if (!column.minWidth) {
    column.minWidth = 80;
  }

  column.realWidth = column.width === undefined ? column.minWidth : column.width;

  return column;
};

//计算宽度
const parseWidth = (width) => {
  if (width !== undefined) {
    width = parseInt(width, 10);
    if (isNaN(width)) {
      width = null;
    }
  }
  return width;
};

//计算最小宽度
const parseMinWidth = (minWidth) => {
  if (minWidth !== undefined) {
    minWidth = parseInt(minWidth, 10);
    if (isNaN(minWidth)) {
      minWidth = 80;
    }
  }
  return minWidth;
};


export {
  columns,
  getDefaultOptions,
  getDefaultColumn,
  parseWidth,
  parseMinWidth
};
